//***********************************************************************
//		Vandelay / AppWindow.cpp
//
//				Copyright (c) 2002-2003 David Blumberg
//						davidblumberg@linux.se
//					Published under the MIT license
//***********************************************************************

#include <Application.h>
#include "App.h"
#include "AppWindow.h"


AppWindow::AppWindow(BRect frame)
		: BWindow(frame, "Vandelay", B_TITLED_WINDOW, B_NOT_RESIZABLE | B_NOT_ZOOMABLE)
{
	rgb_color bgcolor = ui_color(B_PANEL_BACKGROUND_COLOR);
	
	// BMenuBar
	fMenuBar = new BMenuBar(Bounds(), "Menubar");
	
	File = new BMenu("File");
	File->AddItem(new BMenuItem("About", new BMessage(FILE_ABOUT)));
	File->AddSeparatorItem();
	File->AddItem(new BMenuItem("Quit", new BMessage(FILE_QUIT), 'Q'));
	fMenuBar->AddItem(File);
	
	AddChild(fMenuBar);
	
	// BView
	BRect ViewRect = Bounds();
	ViewRect.top = fMenuBar->Bounds().bottom + 1;
	fView = new BView(ViewRect, "View", B_FOLLOW_ALL, B_WILL_DRAW);
	fView->SetViewColor(bgcolor);
	AddChild(fView);
	
	// Categories BMenuField
	CatPopMenu = new BPopUpMenu("Categories");
	CatPopMenu->AddItem(new BMenuItem("Area", new BMessage(CAT_AREA)));
	CatPopMenu->AddItem(new BMenuItem("Distance", new BMessage(CAT_DIST)));
	CatPopMenu->AddItem(new BMenuItem("Power", new BMessage(CAT_POWER)));
	CatPopMenu->AddItem(new BMenuItem("Pressure", new BMessage(CAT_PRESS)));
	CatPopMenu->AddItem(new BMenuItem("Temperature", new BMessage(CAT_TEMP)));
	CatPopMenu->AddItem(new BMenuItem("Volume", new BMessage(CAT_VOLUME)));
	CatPopMenu->AddItem(new BMenuItem("Weight", new BMessage(CAT_WEIGHT)));
	
	Categories = new BMenuField(BRect(0.0, 0.0, 120.0, 0.0), "Categories field", NULL, CatPopMenu);
	
	// BBox
	BRect BoxRect = fView->Bounds();
	BoxRect.top += 10;
	BoxRect.bottom -= 30;
	BoxRect.InsetBy(10, 0);
	fBox = new BBox(BoxRect, "Box", B_FOLLOW_ALL);
	fBox->SetLabel(Categories);
	fView->AddChild(fBox);
	
	// Input BTextControl
	Input = new BTextControl(BRect(0.0, 0.0, 110.0, 0.0).OffsetToCopy(10.0, 35.0), "Input", "Convert:", NULL, NULL);
	Input->SetAlignment(B_ALIGN_RIGHT, B_ALIGN_LEFT);
	Input->SetDivider(45.0);
	Input->SetEnabled(false);
	fBox->AddChild(Input);
	
	// From BMenuField
	FromMenu = new BPopUpMenu("<Select Unit>");
	
	FromField = new BMenuField(BRect(0.0, 0.0, 140.0, 0.0).OffsetToCopy(132.0, 33.0), "From", "From:", FromMenu);
	FromField->SetAlignment(B_ALIGN_RIGHT);
	FromField->SetDivider(32.0);
	FromField->SetEnabled(false);
	fBox->AddChild(FromField);
	
	// To BMenuField
	ToMenu = new BPopUpMenu("<Select Unit>");
	
	ToField = new BMenuField(BRect(0.0, 0.0, 140.0, 0.0).OffsetToCopy(132.0, 68.0), "To", "To:", ToMenu);
	ToField->SetAlignment(B_ALIGN_RIGHT);
	ToField->SetDivider(32.0);
	ToField->SetEnabled(false);
	fBox->AddChild(ToField);
	
	// Output BTextControl
	Output = new BTextControl(BRect(0.0, 0.0, 160.0, 0.0).OffsetToCopy(43.0, 130.0), "Output", "Result:", NULL, NULL);
	Output->SetAlignment(B_ALIGN_RIGHT, B_ALIGN_LEFT);
	Output->SetDivider(35.0);
	Output->SetEnabled(false);
	fBox->AddChild(Output);
	
	// Detail BStringView
	fStringView = new BStringView(BRect(0.0, 0.0, 280.0, 13.0).OffsetToCopy(10.0, 180.0), "Detail view", "Choose a category above...");
	fStringView->SetAlignment(B_ALIGN_CENTER);
	fView->AddChild(fStringView);
}

//----------------------------------------------------------------------

AppWindow::~AppWindow() {
}

bool AppWindow::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return(true);
}

//----------------------------------------------------------------------

void AppWindow::MessageReceived(BMessage *message)
{
	switch(message->what)
	{
		case FILE_ABOUT:
		{
			About = new BAlert("About", "Vandelay 1.5\n\nCopyright (c) 2002-2003 David Blumberg.\ndavidblumberg@linux.se", "Charming", NULL, NULL, B_WIDTH_FROM_WIDEST);
			About->Go();
		}
		break;
		case FILE_QUIT:
			QuitRequested();
		break;
		case CAT_AREA:
		{
			EnableViews();
			category = 0;
			convertEnable = false;
			
			// Remove old menuitems if present
			if(FromMenu->CountItems() > 0)
			{
				for(itemIndex = FromMenu->CountItems(); itemIndex >= 0; itemIndex--)
				{
					FromMenu->RemoveItem(itemIndex);
					ToMenu->RemoveItem(itemIndex);
				}
			}
			
			// Add new menuitems
			FromMenu->AddItem(new BMenuItem("Acre", new BMessage(UNIT_FROM)));
			FromMenu->AddItem(new BMenuItem("Centimeter²", new BMessage(UNIT_FROM)));
			FromMenu->AddItem(new BMenuItem("Hectare", new BMessage(UNIT_FROM)));
			FromMenu->AddItem(new BMenuItem("Inch²", new BMessage(UNIT_FROM)));
			FromMenu->AddItem(new BMenuItem("Meter²", new BMessage(UNIT_FROM)));
			FromMenu->AddItem(new BMenuItem("Yard²", new BMessage(UNIT_FROM)));
			
			ToMenu->AddItem(new BMenuItem("Acre", new BMessage(UNIT_TO)));
			ToMenu->AddItem(new BMenuItem("Centimeter²", new BMessage(UNIT_TO)));
			ToMenu->AddItem(new BMenuItem("Hectare", new BMessage(UNIT_TO)));
			ToMenu->AddItem(new BMenuItem("Inch²", new BMessage(UNIT_TO)));
			ToMenu->AddItem(new BMenuItem("Meter²", new BMessage(UNIT_TO)));
			ToMenu->AddItem(new BMenuItem("Yard²", new BMessage(UNIT_TO)));
		}
		break;
		case CAT_DIST:
		{
			EnableViews();
			category = 1;
			convertEnable = false;
			
			// Remove old menuitems if present
			if(FromMenu->CountItems() > 0)
			{
				for(itemIndex = FromMenu->CountItems(); itemIndex >= 0; itemIndex--)
				{
					FromMenu->RemoveItem(itemIndex);
					ToMenu->RemoveItem(itemIndex);
				}
			}
			
			// Add new menuitems
			FromMenu->AddItem(new BMenuItem("Foot", new BMessage(UNIT_FROM)));
			FromMenu->AddItem(new BMenuItem("Kilometer", new BMessage(UNIT_FROM)));
			FromMenu->AddItem(new BMenuItem("Meter", new BMessage(UNIT_FROM)));
			FromMenu->AddItem(new BMenuItem("Mile(US)", new BMessage(UNIT_FROM)));
			
			ToMenu->AddItem(new BMenuItem("Foot", new BMessage(UNIT_TO)));
			ToMenu->AddItem(new BMenuItem("Kilometer", new BMessage(UNIT_TO)));
			ToMenu->AddItem(new BMenuItem("Meter", new BMessage(UNIT_TO)));
			ToMenu->AddItem(new BMenuItem("Mile(US)", new BMessage(UNIT_TO)));
		}
		break;
		case CAT_POWER:
		{
			EnableViews();
			category = 2;
			convertEnable = false;
			
			// Remove old menuitems if present
			if(FromMenu->CountItems() > 0)
			{
				for(itemIndex = FromMenu->CountItems(); itemIndex >= 0; itemIndex--)
				{
					FromMenu->RemoveItem(itemIndex);
					ToMenu->RemoveItem(itemIndex);
				}
			}
			
			// Add new menuitems
			FromMenu->AddItem(new BMenuItem("Horsepower", new BMessage(UNIT_FROM)));
			FromMenu->AddItem(new BMenuItem("Kilowatt", new BMessage(UNIT_FROM)));
			
			ToMenu->AddItem(new BMenuItem("Horsepower", new BMessage(UNIT_TO)));
			ToMenu->AddItem(new BMenuItem("Kilowatt", new BMessage(UNIT_TO)));
		}
		break;
		case CAT_PRESS:
		{
			EnableViews();
			category = 3;
			convertEnable = false;
			
			// Remove old menuitems if present
			if(FromMenu->CountItems() > 0)
			{
				for(itemIndex = FromMenu->CountItems(); itemIndex >= 0; itemIndex--)
				{
					FromMenu->RemoveItem(itemIndex);
					ToMenu->RemoveItem(itemIndex);
				}
			}
			
			// Add new menuitems
			FromMenu->AddItem(new BMenuItem("Bar", new BMessage(UNIT_FROM)));
			FromMenu->AddItem(new BMenuItem("Pascal(Pa)", new BMessage(UNIT_FROM)));
			FromMenu->AddItem(new BMenuItem("PSI", new BMessage(UNIT_FROM)));
			
			ToMenu->AddItem(new BMenuItem("Bar", new BMessage(UNIT_TO)));
			ToMenu->AddItem(new BMenuItem("Pascal(Pa)", new BMessage(UNIT_TO)));
			ToMenu->AddItem(new BMenuItem("PSI", new BMessage(UNIT_TO)));
		}
		break;
		case CAT_TEMP:
		{
			EnableViews();
			category = 4;
			convertEnable = false;
			
			// Remove old menuitems if present
			if(FromMenu->CountItems() > 0)
			{
				for(itemIndex = FromMenu->CountItems(); itemIndex >= 0; itemIndex--)
				{
					FromMenu->RemoveItem(itemIndex);
					ToMenu->RemoveItem(itemIndex);
				}
			}
			
			// Add new menuitems
			FromMenu->AddItem(new BMenuItem("Celsius", new BMessage(UNIT_FROM)));
			FromMenu->AddItem(new BMenuItem("Fahrenheit", new BMessage(UNIT_FROM)));
			FromMenu->AddItem(new BMenuItem("Kelvin", new BMessage(UNIT_FROM)));
			FromMenu->AddItem(new BMenuItem("Rankine", new BMessage(UNIT_FROM)));
			
			ToMenu->AddItem(new BMenuItem("Celsius", new BMessage(UNIT_TO)));
			ToMenu->AddItem(new BMenuItem("Fahrenheit", new BMessage(UNIT_TO)));
			ToMenu->AddItem(new BMenuItem("Kelvin", new BMessage(UNIT_TO)));
			ToMenu->AddItem(new BMenuItem("Rankine", new BMessage(UNIT_TO)));
		}
		break;
		case CAT_VOLUME:
		{
			EnableViews();
			category = 5;
			convertEnable = false;
			
			// Remove old menuitems if present
			if(FromMenu->CountItems() > 0)
			{
				for(itemIndex = FromMenu->CountItems(); itemIndex >= 0; itemIndex--)
				{
					FromMenu->RemoveItem(itemIndex);
					ToMenu->RemoveItem(itemIndex);
				}
			}
			
			// Add new menuitems
			FromMenu->AddItem(new BMenuItem("Dm³", new BMessage(UNIT_FROM)));
			FromMenu->AddItem(new BMenuItem("Gallon(US)", new BMessage(UNIT_FROM)));
			FromMenu->AddItem(new BMenuItem("Litre", new BMessage(UNIT_FROM)));
			
			ToMenu->AddItem(new BMenuItem("Dm³", new BMessage(UNIT_TO)));
			ToMenu->AddItem(new BMenuItem("Gallon(US)", new BMessage(UNIT_TO)));
			ToMenu->AddItem(new BMenuItem("Litre", new BMessage(UNIT_TO)));
		}
		break;
		case CAT_WEIGHT:
		{
			EnableViews();
			category = 6;
			convertEnable = false;
			
			// Remove old menuitems if present
			if(FromMenu->CountItems() > 0)
			{
				for(itemIndex = FromMenu->CountItems(); itemIndex >= 0; itemIndex--)
				{
					FromMenu->RemoveItem(itemIndex);
					ToMenu->RemoveItem(itemIndex);
				}
			}
			
			// Add new menuitems
			FromMenu->AddItem(new BMenuItem("Gram", new BMessage(UNIT_FROM)));
			FromMenu->AddItem(new BMenuItem("Kilogram", new BMessage(UNIT_FROM)));
			FromMenu->AddItem(new BMenuItem("Pound", new BMessage(UNIT_FROM)));
			
			ToMenu->AddItem(new BMenuItem("Gram", new BMessage(UNIT_TO)));
			ToMenu->AddItem(new BMenuItem("Kilogram", new BMessage(UNIT_TO)));
			ToMenu->AddItem(new BMenuItem("Pound", new BMessage(UNIT_TO)));
		}
		break;
		
		case UNIT_FROM:
		{
			message->FindInt32("index", &from_index);
			Convert();
			convertEnable = true;
		}
		break;
		case UNIT_TO:
		{
			message->FindInt32("index", &to_index);
			Convert();
			convertEnable = true;
		}
		break;
		
		default:
			BWindow::MessageReceived(message);
		break;
	}
}

//----------------------------------------------------------------------

// Enable dissabled views and change StringView text
void AppWindow::EnableViews()
{
	if(Input->IsEnabled() == false)
	{
		Input->SetEnabled(true);
		FromField->SetEnabled(true);
		ToField->SetEnabled(true);
		Output->SetEnabled(true);
		fStringView->SetText("...now enter a value to start converting.");
	}
}
